/* --------------------------- */
/* JS extensions and overrides */
/* --------------------------- */
// Date related functions and defines
Date.now = Date.now || function () { return +new Date; }; // no Date.now in IE<9 
Date.prototype.format = function (date) {
    var o = {
        "M+": this.getMonth() + 1,
        "d+": this.getDate(),
        "h+": this.getHours(),
        "m+": this.getMinutes(),
        "s+": this.getSeconds(),
        "q+": Math.floor((this.getMonth() + 3) / 3),
        "S": this.getMilliseconds()
    }

    if (/(y+)/.test(date)) {
        date = date.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
    }
    for (var k in o) {
        if (new RegExp("(" + k + ")").test(date)) {
            date = date.replace(RegExp.$1, RegExp.$1.length == 1 ? o[k] : ("00" + o[k]).substr(("" + o[k]).length));
        }
    }

    return date;
}
// Need this for IE8
if (!Date.prototype.toISOString) {
    (function() {
        function pad(number) {
            var r = String(number);
            if ( r.length === 1 ) {
                r = '0' + r;
            }
            return r;
        }
        Date.prototype.toISOString = function() {
            return this.getUTCFullYear()
                + '-' + pad( this.getUTCMonth() + 1 )
                + '-' + pad( this.getUTCDate() )
                + 'T' + pad( this.getUTCHours() )
                + ':' + pad( this.getUTCMinutes() )
                + ':' + pad( this.getUTCSeconds() )
                + '.' + String( (this.getUTCMilliseconds()/1000).toFixed(3) ).slice( 2, 5 )
                + 'Z';
        };
    }());
}

// Extension for generic JSON object
if (!("JSON" in window)) {	
	window.JSON = { // warning: this is just a subset of real JSON
		hextable: ['0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'],
		encode: function(obj) {
			var i;
			var ret;
			var ch;
			if(obj.constructor == Array) {
				ret = '[';
				for(i=0; i<obj.length; ++i) {
					if(i >= 1) ret += ',';
					ret += this.encode(obj[i]);
				}
				return ret + ']';
			} else if(obj.constructor == String) {
				ret = '"';
				for(i=0; i<obj.length; ++i) {
					ch = obj.charCodeAt(i);
					if(ch != 34 && ch != 39 && ch != 92 && ch >= 32 && ch <= 126) {
						ret += obj.charAt(i);
					} else {
						ret += '\\u' + this.hextable[(ch >> 12) & 0xf] + this.hextable[(ch >> 8) & 0xf] + this.hextable[(ch >> 4) & 0xf] + this.hextable[ch & 0xf];
					}
				}
				return ret + '"';
			} else if(obj.constructor == Number && ('' + obj) == ('' + Math.round(obj))) {
				return ''+obj;
			} else if(obj.constructor == Object) {
				ret = '{';
				for(i in obj) {
					if(ret.length > 1) ret += ',';
					ret += this.encode(i) + ':' + this.encode(obj[i]);
				}
				ret += '}';
				return ret;
			} else {
				throw "unsupported type";
			}
		}
	}	
    window.JSON.stringify = window.JSON.encode;
}

// This is JS String object extension with startsWith, endsWith and trim functions
if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function (str) {
        return this.slice(0, str.length) == str;
    };
}
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function (str) {
        return this.slice(-str.length) == str;
    };
}
if (typeof String.prototype.trim != 'function') {
	String.prototype.trim = function () {
	   return this.replace(/^\s+|\s+$/g, '');
	}
}


/* ------------------------------ */
/* jQuery extensions and plugins */
/* ------------------------------ */
(function () {
	/**
	* Use by adding these properties to any element:
	*  class="tooltip" (or add class tooltip to existing class declaration)
	*  title="[tooltip content]" (watch for " in the content - encode as &quot;)
	*  (optional) data-position="n" (possible values: n, ne, e, se, s, sw, w, nw)
	*  (optional) data-class="" (will add classes to new tooltip div)
	*
	* If the element has an existing id, the id of the tooltip will be [id]-popup. If that element exists
	* it will be used and the title is not needed on the .tooltip element.
	**/
	var i = 0;
	function genId() { return "tooltip-popup-" + (++i); }
	$.fn.extend({
	    'tooltip': function () {
            var $body = $('body');
	        return $(this).each(function () {
	            /**
	            * change the delays to achieve the desired effect:
	            * - too fast and it'll bother you when moving the mouse
	            * - too slow and you won't notice there's a tooltip as you'll move away before it shows.
	            **/
	            var showDelay = 1000,
					hideDelay = showDelay,
	            // don't change from here on if you don't know what you're doing
					tooltipClass = 'tooltiptext', // this class should be styled in CSS
					onAfterDraw = null, // this callback function is called after popup is drawn
					onInitDraw = false, // does it draw tooltip when on init, default is false
					onAfterShow = null, // this callback function is called after popup is showed
					defaultPosition = 'n',
					availablePositions = { 'n': 'n', 'ne': 'ne', 'e': 'e', 'se': 'se', 's': 's', 'sw': 'sw', 'w': 'w', 'nw': 'nw' }, // available positions
	            // don't change from here on, even if you know what you're doing :)
					shown = false,
					$e = $(this),
					id = $e.attr('id') ? $e.attr('id') + '-popup' : genId(),
					txt = $e.attr('title') || '',
					$t = $(id).hide(), showTimer, hideTimer;

                    $e.addClass('selected');

	            if ($e.data('class')) tooltipClass = $e.data('class');
	            if ($e.data('onafterdraw')) onAfterDraw = $e.data('onafterdraw');
	            if ($e.data('onaftershow')) onAfterShow = $e.data('onaftershow');
	            if ($e.data('oninitdraw')) onInitDraw = $e.data('oninitdraw');
	            if ('' === txt && $e.attr('id') && $('#' + $e.attr('id') + '-popup').length) txt = $('#' + id).html();
	            if (!txt && !$t.length) return;
	            $e.attr('title', '');
	            function delayedShow(ev) {
	                if (shown) return;
	                clearTimeout(showTimer);
	                draw();
	                var _this = this;
	                showTimer = setTimeout(function () {
	                    show.call(_this, ev);
	                }, showDelay);
	            }
	            function cancelShow() {
	                if (shown) return;
	                clearTimeout(showTimer);
	            }
	            function delayedHide() {
	                if (!shown) return;
	                clearTimeout(hideTimer);
	                hideTimer = setTimeout(hide, hideDelay);
	            }
	            function cancelHide() {
	                if (!shown) return;
	                clearTimeout(hideTimer);
	            }
	            function show(ev) {
	                if (shown) return;
	                try { // IE has problems with stopPropagation on a delayed event
	                    ev.stopPropagation();
	                } catch (er) { }
	                clearTimeout(showTimer);
	                draw();
	                position(ev);
	                $t.show();
	                shown = true;

	                if (null !== onAfterShow) {
	                    eval(onAfterShow + '(id)');
	                }
	            }
	            function hide() {
	                if (!shown) return;
	                var ax = $t.data('animate-left'),
						ay = $t.data('animate-top');
	                $t
						.animate({
							'top': '-=' + ay,
							'left': '-=' + ax,
							'opacity': .2
						}, 50, function () {
							$t.hide();
						});
	                shown = false;
                    $e.removeClass('selected');
	            }
	            function stop(ev) {
	                ev.stopPropagation();
	            }
	            function draw() {
	                if (!$t.length) {
	                    $t = $('<div />');
	                    $t
							.addClass(tooltipClass)
							.attr('id', id)
							.html(txt)
							.appendTo($body)
							.hide();
	                }
	                $t
						.css({
							'position': 'absolute',
							'top': 0,
							'left': 0,
							'z-index': 2000,
                            'min-width': $e.outerWidth()
						})
						.bind({
							'mouseover': cancelHide,
							'mouseout': delayedHide,
							'click': stop
						})
						.data({
							'width': $t.outerWidth(),
							'height': $t.outerHeight()
						});
	                if ($e.data('width') && $e.data('width') == 'inherit') {
	                    $t.width($('#' + id).outerWidth());
	                }

	                if (null !== onAfterDraw) {
	                    eval(onAfterDraw + '(id)');
	                }
	            }
	            function resolvePosition(str, def) {
	                var p = availablePositions[str.toLowerCase()] || def,
						o = { n: false, s: false, e: false, w: false };
	                if (p.indexOf('n') >= 0) o.north = true;
	                else if (p.indexOf('s') >= 0) o.south = true;
	                if (p.indexOf('e') >= 0) o.east = true;
	                else if (p.indexOf('w') >= 0) o.west = true;
	                return o;
	            }
	            function position(ev) {
	                var w = $t.data('width'),
						h = $t.data('height'),
						p = resolvePosition($e.data('position') || '', defaultPosition),
						bw = !p.west ? $e.outerWidth() : 0,
						bh = !p.north ? $e.outerHeight() : 0,
						o = $e.offset(),
						nx, ny, ay = 0, ax = 0;
	                // north - south
	                if (p.north) {
	                    ny = o.top - h + 5;
	                    ay = -1;
	                } else if (p.south) {
	                    ny = o.top + bh - 5;
	                    ay = 1;
	                } else {
	                    ny = o.top - h / 2 + bh / 2;
	                }
	                // east - west
	                if (p.east) {
	                    nx = o.left + bw - 5;
	                    if ($e.data('float') && $e.data('float') == 'left') {
	                        nx = o.left + bw - 10 - $e.outerWidth();
	                    }
	                    ax = 1;
	                } else if (p.west) {
	                    nx = o.left - w + 5;
	                    if ($e.data('float') && $e.data('float') == 'right') {
	                        nx = o.left + $e.outerWidth() - w + 10;
	                    }
	                    ax = -1;
	                } else {
	                    nx = o.left - w / 2 + bw / 2;
	                }
	                ax *= 10;
	                ay *= 10;
	                $t
						.css({
							'left': nx,
							'top': ny,
							'opacity': .2
						})
						.data({ 'animate-left': ax, 'animate-top': ay })
						.animate({
							'top': '+=' + ay,
							'left': '+=' + ax,
							'opacity': 1
						}, 100);
	            }
	            $e
					.bind({
						'mouseover': delayedShow,
						'mouseout': cancelShow,
						'click': show
					})
					.bind({
						'mouseover': cancelHide,
						'mouseout': delayedHide
					});
	            if ($e.css('position') === 'static') {
	                $e.css('position', 'relative');
	            }
	            $body.bind('click', hide);

	            if (true == onInitDraw) {
	                draw();
	            }
	        });
	    }
	});
	$('.tooltip').tooltip();

    /**
     * Function executes full text search on defined item(s)
     *
     * @param object options override options if default are not working
     *               element - which child elements to show/hide on full-text-search
     *               timeOut - delay before search starts (200ms is default)
     *               form - true if search form is wrapped inside form and you would like to disable form post on enter
     *
     * @return string Translated string
     *
     */
    var updateFilterInterval = null, updateListInterval = null;
    $.fn.fts = function(options) {
 
        // This is the easiest way to have default options.
        var settings = $.extend({
            element : '.row',
            timeOut : 200,
            form : false
        }, options), $obj = this;

        // Disable form submit
        if (true === settings.form) {
            $obj.closest('form').bind('submit', function() { return false; });
        }

        // Do search
        return this.bind('keyup', function (event) {
            if (null !== updateListInterval) clearTimeout(updateListInterval);
            
            updateListInterval = setTimeout(function () { 
                var searchString = $.trim($obj.val().replace(/\s{2,}/g, ' ').toLowerCase()),
                searchTerms = searchString.split(' '),
                found = false,
                fullText = '';

            if (searchString.length === 0) {
                $(settings.element).show();
            }

            $(settings.element).each(function (index, value) {
                found = true;
                fullText = $(this).data('fulltext').toLowerCase();
                
                for (var i = 0; i < searchTerms.length; i++) {
                    if (fullText.indexOf(searchTerms[i]) === -1) found = false;
                }

                if (!found && $(this).is(':visible')) $(this).hide();
                if (found && $(this).is(':hidden')) $(this).show();
            });
            }, settings.timeOut);
        });
    };
} ());


/* --------- */
/* FUNCTIONS */
/* --------- */

// Add this line to each translations object in every file that inlcudes common.js
//'No data from API':{{translate_js|No data from API}}
var translations = {
    'No data from API':'No data from API',
};

/**
 * Function gets translated string from translations object; if none found original string is returned
 *
 * @param string in_text Text you want to translate
 *
 * @return string Translated string
 *
 */
function translate(in_text) {
    if (in_text in translations) {
        return translations[in_text];
    }
    return in_text;
}

/**
 * This function is used for webapi2 calls
 * NOTE: If global sessionId and languageCode variables are defined they will be used at api callss
 *
 * @param string method which API method to envoke
 * @param JSON reqdata_raw data required by API method
 * @param function ok_func callback function if call succeeds
 * @param function err_func callback function if call fails
 *
 */
function callwebapi(method, reqdata_raw, ok_func, err_func, async) {
    var reqdata = JSON.stringify(reqdata_raw, null),
        url = '/webapi2?he=JSON1&method=' + encodeURI(method);
	
	if ('undefined' === typeof(async) || null === async) {
		async = false;
	}
    
    if ('undefined' !== typeof(sessionId) && '' !== sessionId && null !== sessionId) {
        url += '&hs=' + sessionId;
    }

    if ('undefined' !== typeof(languageCode) && '' !== languageCode && null !== languageCode) {
        url += '&hl=' + languageCode;
    }

    // using POST so we can send larger data
    // we cannot make sendEmail - size limit reached (IE 2k)
    // we will not add: '&hedata=' + encodeURI(reqdata) + '&jsonp=?' 
    $.ajax({    
            url: url,
            dataType: 'json',
            type:'POST',
            data: reqdata,
			async: async,
//            jsonp:'jsonp',
//            crossDomain: true,
            success: function(data) {
                if (typeof(data) != 'object' || typeof(data.result) != 'object') {
                    executeError(translate('No data from API'));
                    return;
                }
                if ('OK' == data.result.code) {
                    if (typeof(data.result.hs) == 'string') {
                        sessionId = data.result.hs;
                    }
                    ok_func(data);
                } else {
                    if ('function' === typeof(err_func)) {
                        err_func(data);
                    } else {
                        executeError(translate('No data from API'));
                    }
                }                
            },
            error: function () {
                if ('function' === typeof(err_func)) {
                    err_func(data);
                } else {
                    executeError(translate('No data from API'));
                }
            }
    });
}

/**
 * This function calls utils/users/query method of webapi2 on conference proxy
 * NOTE: It sets value to global I variable
 *
 * @param function func callback function
 *
 */
function whoAmI(func) {
    callwebapi('utils/users/query/1', {}, function(data) {           
            // Get info for list
            I = data.data.I;
            I.default_domain = data.data.default_domain;
            if ('undefined' !== typeof(func)) setTimeout(func, 10);
        }
    );
}

/**
 * Function handles output for error message. All functions call this one when error occurs.
 * NOTE: Function is wrapped in try/catch in case error function fails
 *
 * @param string errorMessage Error message string
 *
 */
function executeError(errorInfo) {
    try {
        if (typeof(errorInfo) === 'object') {
           // If session timeoud out, do hard refresh
            if ('IMPLEMENTATION_ERROR' === errorInfo.code) {
                if (typeof(errorInfo.errors) === 'object' && '/webapi/invalid-session' === errorInfo.errors['_']) {
                    location.href = location.href;
                } else if (typeof(errorInfo.errors) === 'object' && '/webapi/not-logged-in' === errorInfo.errors['_']) {
                    location.href = '/users/main/login.html';
                }
            } else {
                if (console) {
                    console.log('ERROR : ' + errorInfo.description);
                } else {
                    alert(errorInfo.description);
                }
            }            
        } else {
            if (console) {
                console.log('ERROR : ' + errorInfo.description);
            } else {
                alert(errorInfo);
            }
        }        
    } catch (e) {
        if (console) {
            console.log('ERROR : ' + errorInfo.description);
        } else {
            alert(e);
        }
    }
}

/**
 * Function sends email using utils/email/1 webapi call
 *
 * @param array recipients Array of recipients' emails
 * @param string subject Email's subject
 * @param string body Email's body/content
 * @param function done_func callback function called when emails are sent
 *
 */
function sendEmail(recipients, subject, body, done_func) {   
    var req = { 'subject' : subject, 
                'body_html' : body,
                'address' : recipients};

    callwebapi('utils/email/1', req, function (data) {
            if (isSet(done_func)) done_func(data);
        }
    );
}



/**
 * Function sends email using utils/email/1 webapi call
 *
 * @param string email Email address you would like to check
 * @param boolean strict Should it be a strict check of email
 *
 * @return boolean true if valid, false if not
 *
 */
function isValideEmailAddress(email, strict) {
    var re = /\S+@\S+\.\S+/;

    if (isSet(strict) && true === strict) {
        re = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    }
    
    return re.test(email);
}

/**
 * Function gets value of query string parameter or returns null if it is not found
 *
 * @param string key query string parameter name
 *
 * @return string null if not found and parameter value if found
 *
 */
function getQuerystring(key)
{
    key = key.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");
    var regex = new RegExp("[\\?&]"+key+"=([^&#]*)");
    var qs = regex.exec(window.location.href);
    if (null !== qs && qs.length > 0) {
        return qs[1];
    } else {
        return null;
    }
}

/**
 * Function returns date string in local format; if locale is not found it will return default format yyyy-MM-dd
 *
 * @param Date date date you would like to display
 *
 * @return string formated date
 *
 */
function getLocaleDateString(date) {
    var formats = { "ar-SA": "dd/MM/yy", "bg-BG": "dd.M.yyyy", "ca-ES": "dd/MM/yyyy", "zh-TW": "yyyy/M/d", "cs-CZ": "d.M.yyyy", "da-DK": "dd-MM-yyyy", "de-DE": "dd.MM.yyyy", "el-GR": "d/M/yyyy", "en-US": "M/d/yyyy", "fi-FI": "d.M.yyyy", "fr-FR": "dd/MM/yyyy", "he-IL": "dd/MM/yyyy", "hu-HU": "yyyy. MM. dd.", "is-IS": "d.M.yyyy", "it-IT": "dd/MM/yyyy", "ja-JP": "yyyy/MM/dd", "ko-KR": "yyyy-MM-dd", "nl-NL": "d-M-yyyy", "nb-NO": "dd.MM.yyyy", "pl-PL": "yyyy-MM-dd", "pt-BR": "d/M/yyyy", "ro-RO": "dd.MM.yyyy", "ru-RU": "dd.MM.yyyy", "hr-HR": "d.M.yyyy", "sk-SK": "d. M. yyyy", "sq-AL": "yyyy-MM-dd", "sv-SE": "yyyy-MM-dd", "th-TH": "d/M/yyyy", "tr-TR": "dd.MM.yyyy", "ur-PK": "dd/MM/yyyy", "id-ID": "dd/MM/yyyy", "uk-UA": "dd.MM.yyyy", "be-BY": "dd.MM.yyyy", "sl-SI": "d.M.yyyy", "et-EE": "d.MM.yyyy", "lv-LV": "yyyy.MM.dd.", "lt-LT": "yyyy.MM.dd", "fa-IR": "MM/dd/yyyy", "vi-VN": "dd/MM/yyyy", "hy-AM": "dd.MM.yyyy", "az-Latn-AZ": "dd.MM.yyyy", "eu-ES": "yyyy/MM/dd", "mk-MK": "dd.MM.yyyy", "af-ZA": "yyyy/MM/dd", "ka-GE": "dd.MM.yyyy", "fo-FO": "dd-MM-yyyy", "hi-IN": "dd-MM-yyyy", "ms-MY": "dd/MM/yyyy", "kk-KZ": "dd.MM.yyyy", "ky-KG": "dd.MM.yy", "sw-KE": "M/d/yyyy", "uz-Latn-UZ": "dd/MM yyyy", "tt-RU": "dd.MM.yyyy", "pa-IN": "dd-MM-yy", "gu-IN": "dd-MM-yy", "ta-IN": "dd-MM-yyyy", "te-IN": "dd-MM-yy", "kn-IN": "dd-MM-yy", "mr-IN": "dd-MM-yyyy", "sa-IN": "dd-MM-yyyy", "mn-MN": "yy.MM.dd", "gl-ES": "dd/MM/yy", "kok-IN": "dd-MM-yyyy", "syr-SY": "dd/MM/yyyy", "dv-MV": "dd/MM/yy", "ar-IQ": "dd/MM/yyyy", "zh-CN": "yyyy/M/d", "de-CH": "dd.MM.yyyy", "en-GB": "dd/MM/yyyy", "es-MX": "dd/MM/yyyy", "fr-BE": "d/MM/yyyy", "it-CH": "dd.MM.yyyy", "nl-BE": "d/MM/yyyy", "nn-NO": "dd.MM.yyyy", "pt-PT": "dd-MM-yyyy", "sr-Latn-CS": "d.M.yyyy", "sv-FI": "d.M.yyyy", "az-Cyrl-AZ": "dd.MM.yyyy", "ms-BN": "dd/MM/yyyy", "uz-Cyrl-UZ": "dd.MM.yyyy", "ar-EG": "dd/MM/yyyy", "zh-HK": "d/M/yyyy", "de-AT": "dd.MM.yyyy", "en-AU": "d/MM/yyyy", "es-ES": "dd/MM/yyyy", "fr-CA": "yyyy-MM-dd", "sr-Cyrl-CS": "d.M.yyyy", "ar-LY": "dd/MM/yyyy", "zh-SG": "d/M/yyyy", "de-LU": "dd.MM.yyyy", "en-CA": "dd/MM/yyyy", "es-GT": "dd/MM/yyyy", "fr-CH": "dd.MM.yyyy", "ar-DZ": "dd-MM-yyyy", "zh-MO": "d/M/yyyy", "de-LI": "dd.MM.yyyy", "en-NZ": "d/MM/yyyy", "es-CR": "dd/MM/yyyy", "fr-LU": "dd/MM/yyyy", "ar-MA": "dd-MM-yyyy", "en-IE": "dd/MM/yyyy", "es-PA": "MM/dd/yyyy", "fr-MC": "dd/MM/yyyy", "ar-TN": "dd-MM-yyyy", "en-ZA": "yyyy/MM/dd", "es-DO": "dd/MM/yyyy", "ar-OM": "dd/MM/yyyy", "en-JM": "dd/MM/yyyy", "es-VE": "dd/MM/yyyy", "ar-YE": "dd/MM/yyyy", "en-029": "MM/dd/yyyy", "es-CO": "dd/MM/yyyy", "ar-SY": "dd/MM/yyyy", "en-BZ": "dd/MM/yyyy", "es-PE": "dd/MM/yyyy", "ar-JO": "dd/MM/yyyy", "en-TT": "dd/MM/yyyy", "es-AR": "dd/MM/yyyy", "ar-LB": "dd/MM/yyyy", "en-ZW": "M/d/yyyy", "es-EC": "dd/MM/yyyy", "ar-KW": "dd/MM/yyyy", "en-PH": "M/d/yyyy", "es-CL": "dd-MM-yyyy", "ar-AE": "dd/MM/yyyy", "es-UY": "dd/MM/yyyy", "ar-BH": "dd/MM/yyyy", "es-PY": "dd/MM/yyyy", "ar-QA": "dd/MM/yyyy", "es-BO": "dd/MM/yyyy", "es-SV": "dd/MM/yyyy", "es-HN": "dd/MM/yyyy", "es-NI": "dd/MM/yyyy", "es-PR": "dd/MM/yyyy", "am-ET": "d/M/yyyy", "tzm-Latn-DZ": "dd-MM-yyyy", "iu-Latn-CA": "d/MM/yyyy", "sma-NO": "dd.MM.yyyy", "mn-Mong-CN": "yyyy/M/d", "gd-GB": "dd/MM/yyyy", "en-MY": "d/M/yyyy", "prs-AF": "dd/MM/yy", "bn-BD": "dd-MM-yy", "wo-SN": "dd/MM/yyyy", "rw-RW": "M/d/yyyy", "qut-GT": "dd/MM/yyyy", "sah-RU": "MM.dd.yyyy", "gsw-FR": "dd/MM/yyyy", "co-FR": "dd/MM/yyyy", "oc-FR": "dd/MM/yyyy", "mi-NZ": "dd/MM/yyyy", "ga-IE": "dd/MM/yyyy", "se-SE": "yyyy-MM-dd", "br-FR": "dd/MM/yyyy", "smn-FI": "d.M.yyyy", "moh-CA": "M/d/yyyy", "arn-CL": "dd-MM-yyyy", "ii-CN": "yyyy/M/d", "dsb-DE": "d. M. yyyy", "ig-NG": "d/M/yyyy", "kl-GL": "dd-MM-yyyy", "lb-LU": "dd/MM/yyyy", "ba-RU": "dd.MM.yy", "nso-ZA": "yyyy/MM/dd", "quz-BO": "dd/MM/yyyy", "yo-NG": "d/M/yyyy", "ha-Latn-NG": "d/M/yyyy", "fil-PH": "M/d/yyyy", "ps-AF": "dd/MM/yy", "fy-NL": "d-M-yyyy", "ne-NP": "M/d/yyyy", "se-NO": "dd.MM.yyyy", "iu-Cans-CA": "d/M/yyyy", "sr-Latn-RS": "d.M.yyyy", "si-LK": "yyyy-MM-dd", "sr-Cyrl-RS": "d.M.yyyy", "lo-LA": "dd/MM/yyyy", "km-KH": "yyyy-MM-dd", "cy-GB": "dd/MM/yyyy", "bo-CN": "yyyy/M/d", "sms-FI": "d.M.yyyy", "as-IN": "dd-MM-yyyy", "ml-IN": "dd-MM-yy", "en-IN": "dd-MM-yyyy", "or-IN": "dd-MM-yy", "bn-IN": "dd-MM-yy", "tk-TM": "dd.MM.yy", "bs-Latn-BA": "d.M.yyyy", "mt-MT": "dd/MM/yyyy", "sr-Cyrl-ME": "d.M.yyyy", "se-FI": "d.M.yyyy", "zu-ZA": "yyyy/MM/dd", "xh-ZA": "yyyy/MM/dd", "tn-ZA": "yyyy/MM/dd", "hsb-DE": "d. M. yyyy", "bs-Cyrl-BA": "d.M.yyyy", "tg-Cyrl-TJ": "dd.MM.yy", "sr-Latn-BA": "d.M.yyyy", "smj-NO": "dd.MM.yyyy", "rm-CH": "dd/MM/yyyy", "smj-SE": "yyyy-MM-dd", "quz-EC": "dd/MM/yyyy", "quz-PE": "dd/MM/yyyy", "hr-BA": "d.M.yyyy.", "sr-Latn-ME": "d.M.yyyy", "sma-SE": "yyyy-MM-dd", "en-SG": "d/M/yyyy", "ug-CN": "yyyy-M-d", "sr-Cyrl-BA": "d.M.yyyy", "es-US": "M/d/yyyy" },
        lang = navigator.language;
    if ('undefined' !== typeof (config.languageCode) && 'undefined' !== typeof (config.countryCode)) {
        lang = config.languageCode + '-' + config.countryCode;
    }
    
    return formats[lang] || 'yyyy-MM-dd';
}

/**
 * Function checks if object is defined in DOM
 * NOTE: JavaScript does not allow checking tree, so checking a.b will cause error if a is undefined
 *
 * @param undefined obj anything you would like to check
 *
 * @return boolean true if object is defined, false otherwise
 *
 */
function isSet(obj) {
    return 'undefined' === typeof(obj) ? false : true;
}

/**
 * Converts milliseconds to friendly timestamp
 *
 * @param int ms milliseconds to convert
 * @param true ago display time from now
 * @param true full display time in full format
 *
 * @return string formated milliseconds
 *
 */
function convertMilliseconds(ms, ago, full) {
	var hours = Math.max(0, Math.floor ( ms / 3600000 )), // 1 Hour = 36000 Milliseconds
    	minutes = Math.max(1, Math.floor (( ms % 3600000) / 60000)), // 1 Minutes = 60000 Milliseconds
	    //seconds = Math.floor ((( ms % 360000) % 60000) / 1000) // 1 Second = 1000 Milliseconds
	    today = new Date(),
        created = new Date(today.getTime() - ms),
        output, lang = window.navigator.userLanguage || window.navigator.language,
	    options = {
	        weekday: "short",
	        year: "numeric",
	        month: "2-digit",
	        day: "numeric"
	    };

    if (!isSet(full)) full = false;

    if (ago) {
        if (hours > 0) {
            output = hours > 1 ? translate('__h__ hours ago') : translate('__h__ hour ago');
        } else {
            output = minutes > 1 ? translate('__m__ minutes ago') : translate('__m__ minute ago');
        }
    } else {
        if (hours > 0) {
            output = hours > 1 ? translate('__h__ hours') : translate('__h__ hour');
        } else {
            output = minutes > 1 ? translate('__m__ minutes') : translate('__m__ minute');
        }
    }
    if (output) {
        output = output.replace('__h__', hours).replace('__m__', minutes);
    }

    if (hours >= 6 && !full) {
        if (today.toLocaleDateString() === created.toLocaleDateString()) {
            output = translate('today');
        } else if (today.toLocaleDateString() !== created.toLocaleDateString() && (today.getTime() - created.getTime()) < (24 * 3600000)) { // one day
            output = translate('yesterday');
        } else {
            //output = created.format(getLocaleDateString());
			output = created.toLocaleDateString();
            var date = new Date();
        }
    }
    
    return output;
}

/**
 * Converts seconds to interval
 *
 * @param int seconds seconds to convert
 *
 * @return string formated seconds (hh:mm:ss)
 *
 */
function formatSeconds(seconds)
{
    var date = new Date(1970,0,1);
    date.setSeconds(seconds);
    return date.toTimeString().replace(/.*(\d{2}:\d{2}:\d{2}).*/, "$1");
}

/**
 * Show message/error/warning in defined items
 * NOTE: must use message-container class in combination with messageType
 *
 * @param string messageString html you would like to display
 * @param string messageType which type of message to display. If you set this to error, then element with classes message-container and error will be displayed
 *
 */
function ShowMessage(messageString, messageType, append, newline) {
    if (!isSet(append) || true !== append) {
        append = false;
    }
	if (null === messageString) $('.message-container').hide(config.animationSpeed);
    if ('' !== messageString && null !== messageString) {
        if (append) {
			if (isSet(newline) && true === newline) messageString += '<br/>';
            $('.message-container.' + messageType).prepend(messageString).show(config.animationSpeed);
        } else {
            $('.message-container.' + messageType).html(messageString).show(config.animationSpeed);
        }
    } else {
        $('.message-container.' + messageType).html('');
    }
}

/**
 * Converts bytes to readable format 
 *
 * @param int bytes bytes to convert
 * @param int precision decimal places
 *
 * @return string formated bytes string
 *
 */
function bytesToSize(bytes, precision)
{  
    var kilobyte = 1024;
    var megabyte = kilobyte * 1024;
    var gigabyte = megabyte * 1024;
    var terabyte = gigabyte * 1024;
   
    if ((bytes >= 0) && (bytes < kilobyte)) {
        return bytes + ' B';
 
    } else if ((bytes >= kilobyte) && (bytes < megabyte)) {
        return (bytes / kilobyte).toFixed(precision) + ' KB';
 
    } else if ((bytes >= megabyte) && (bytes < gigabyte)) {
        return (bytes / megabyte).toFixed(precision) + ' MB';
 
    } else if ((bytes >= gigabyte) && (bytes < terabyte)) {
        return (bytes / gigabyte).toFixed(precision) + ' GB';
 
    } else if (bytes >= terabyte) {
        return (bytes / terabyte).toFixed(precision) + ' TB';
 
    } else {
        return bytes + ' B';
    }
}

/**
 * Function removes all character from session code except - and 0-9
 *
 * @param string sessionCode Session code to trim
 *
 * @return string cleaned session code
 *
 */
function TrimSessionCode(sessionCode) {
    return null !== sessionCode ? sessionCode.replace(/[^0-9\-]+/g, '') : null;
}

/**
 * Function executes function after timeout interval and calls update function every update interval
 *
 * @param integer time Seconds before complete function is called
 * @param function update Function to call every 1000ms
 * @param function complete Function to call when time expires
 *
 */
function timer(time, update, complete) {
	update(time);
    var tmval = time;
    interval = setInterval(function () {
        tmval --;	
        if( tmval <= 0) {
            clearInterval(interval);
            complete();
        } else {
			update(tmval);
		}
    }, 1000); // the smaller this number, the more accurate the timer will be
}

/**
 * Function takes data object and template id and compiles it into html
 *
 * @param string str template id
 * @param string data object with variables used in template
 *
 * @return string html compiled by function through template
 *
 */
 (function () {
    this.tmpl = function tmpl(str, data) {
        var fn = !/\W/.test(str) ? tmpl(document.getElementById(str).innerHTML) :
            new Function("obj",
                "var p=[],print=function(){p.push.apply(p,arguments);};with(obj){p.push('" +
                str
                    .replace(/[\r\t\n]/g, " ")
                    .split("<%").join("	")
                    .replace(/((^|%>)[^\t]*)'/g, "$1\r")
                    .replace(/\t=(.*?)%>/g, "',$1,'")
                    .split("	").join("');")
                    .split("%>").join("p.push('")
                    .split("\r").join("\\'") +
                "');}return p.join('');");
   
        return data ? fn( data ) : fn;
    };
})();
